package org.ow2.odis.plugin;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.admin.IPlugin;
import org.ow2.odis.domain.Domain;
import org.ow2.odis.exception.OdisFactoryException;
import org.ow2.odis.jmx.JMX;
import org.ow2.odis.services.IFactoryService;

/**
 * The Class JarLoaderPlugin.
 */
public class JarLoaderPlugin implements IPlugin, IFactoryService, JarLoaderPluginMBean {

    /** The list of class name created. */
    List lsClassNameCreated = new ArrayList();
    /** The list of jars path. */
    List lsJars = new LinkedList();
    /** The Constant JarLoaderPluginName. */
    static final String JarLoaderPluginName = "JarLoader";
    /** The old factory. */
    IFactoryService oldFactory = null;
    /** The Constant LOGGER. */
    static final Logger LOGGER = Monolog.initialize().getLogger(JarLoaderPlugin.class.getName());

    /**
     * 
     * @param jarName
     * @see JarLoaderPluginMBean
     */
    public void addJar(String jarName) throws FileNotFoundException {
        File file = new File(jarName);
        if (!file.exists()) {
            throw new FileNotFoundException("unable to find this jar" + jarName);
        }
        lsJars.add(jarName);        
    }

    /**
     * Instantiates a new jar loader plugin.
     */
    public JarLoaderPlugin() {
        JMX.getInstance().registerMBean(getName(), this);
    }

    /**
     * Gets the instance of.
     * @param className the class name
     * @return the instance of
     * @see com.francetelecom.odis.component.loader.IFactory#getInstanceOf(java.lang.String, java.util.Properties)
     */
    public Object getInstanceOf(String className, String version) {
        Object result = load(className);
        if (result==null){
            try {
                result = Class.forName(className);
            } catch (ClassNotFoundException e1) {
                LOGGER.log(BasicLevel.WARN,"NO FILE");
            }
        }
        return result;
    }

    /**
     * Load.
     * @param className the class name
     * @return the object or null if the class is not found    
     */
    private Object load(String className) {
        Iterator itjar = lsJars.iterator();
        Object result = null;
        while (itjar.hasNext() && result == null) {
            String path = (String) itjar.next();
            try {
                File file = new File(path);
                URL url;
                url = file.toURL();
                URLClassLoader loader = new URLClassLoader(new URL[]{url},this.getClass().getClassLoader());                
                Class clazz = loader.loadClass(className);
                result = clazz.newInstance();
                lsClassNameCreated.add(className);
            } catch (Exception e) {
                // c'est normal
            }
        }
        return result;
    }

    private void autoLoad(){
        autoLoad("lib");
    
    }
    private void autoLoad(String arg){
        File directory= new File(arg);
        if (directory.isDirectory()){
            File[] ls = directory.listFiles();            
            for (int i=0;i<ls.length;i++){
                autoLoad(ls[i].getAbsolutePath());
            }
            
        } else{
            try {
                    if (arg.toLowerCase().endsWith(".jar")){                        
                        LOGGER.log(BasicLevel.INFO,arg+" is added to classpath");
                        addJar(arg);
                    }
                    else {
                        LOGGER.log(BasicLevel.WARN,arg+" is not added to classpath");
                    }
                } catch (FileNotFoundException ex) {
                    LOGGER.log(BasicLevel.ERROR,"File"+arg+" doesn't exist");
                }
        }        
    }
    /**
     * Bind.
     * @see com.francetelecom.odis.component.admin.IPlugin#bind()
     */
    public void bind() {
        oldFactory = Domain.getInstance().getFactoryInstance();                
        Domain.getInstance().setFactoryInstance(this);
        autoLoad();
    }

    /**
     * Gets the description.
     * @return the description
     * @see com.francetelecom.odis.component.admin.IPlugin#getDescription()
     */
    public String getDescription() {
        return null;
    }

    /**
     * Gets the name.
     * @return the name
     * @see com.francetelecom.odis.component.admin.IPlugin#getName()
     */
    public String getName() {
        return JarLoaderPluginName;
    }

    /**
     * Unbind the plugin.
     * @see com.francetelecom.odis.component.admin.IPlugin#unbind()
     */
    public void unbind() {
        Domain.getInstance().setFactoryInstance(oldFactory);
    }

    /**
     * Gets the instance of a class.
     * @param className the class name
     * @return the instance of
     * @throws OdisFactoryException the odis factory exception
     * @see com.francetelecom.odis.component.loader.IFactory#getInstanceOf(java.lang.String)
     */
    public Object getInstanceOf(String className) throws OdisFactoryException {
        return getInstanceOf(className, null);
    }

    /**
     * Gets the name of added classes.
     * @return the name of added classes
     * @see com.francetelecom.odis.component.loader.IFactory#getNameOfAddedClasses()
     */
    public String[] getNameOfAddedClasses() {
        String[] result = new String[lsClassNameCreated.size()];
        lsClassNameCreated.toArray(result);
        return result;
    }

    /**
     * 
     * @param jarName
     */
    public void removeJar(String jarName) {
        lsJars.remove(jarName);
    }

    /**
     * 
     * @return
     */
    public String getListJar() {
        StringBuffer result = new StringBuffer("");
        Iterator itPlugins = lsJars.iterator();
        int i = 0;
        while (itPlugins.hasNext()) {
            String entry = (String) itPlugins.next();
            if (entry != null) {
                result.append("[");
                result.append(i++);
                result.append("] ");
                result.append(entry);
                result.append(System.getProperty("line.separator"));
            }
        }
        return result.toString();
    }
}
