package org.ow2.odis.plugin;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.ow2.odis.admin.IPlugin;
import org.ow2.odis.jmx.JMX;
import org.ow2.odis.stat.IStat;
import org.ow2.odis.stat.StatProvider;
import org.ow2.odis.model.AbstractAttribute;


/**
 * The Class Stat.
 */
public class Stat implements IStat, IPlugin, StatMBean {
    private static final String NAME = "Statistique";

    /** Constant. */
    public static final String EOL = System.getProperty("line.separator");

    IStat oldStat = null;

    Map mapAttributeName = new HashMap();

    Map mapAttributeValue = new HashMap();

    String strTracker = "";

    long tracker = 0;

    /**
     * Instantiates a new stat.
     */
    public Stat() {
    }
    
    public long getNanoTime(){
        return System.nanoTime();
    }

    /**
     * Probe.
     * @param attribute the attribute
     * @param value the value
     */
    public void probe(AbstractAttribute attribut, long value) {
        Long moy = (Long) mapAttributeName.get(attribut);
        Long moyenne;
        if (moy == null) {
            moyenne = new Long(value);
        } else {
            moyenne = new Long((moy.longValue() + value) / 2);
        }
        mapAttributeName.put(attribut, moyenne);
        if (attribut.getPName().equals(strTracker)) {
            tracker = (tracker + value) / 2;
        }
    }

    public String[] getStat() {
        Collection col = mapAttributeName.keySet();
        String[] buffers = new String[col.size()];
        Iterator it = col.iterator();
        int i = 0;
        while (it.hasNext()) {
            StringBuffer buffer = new StringBuffer();
            AbstractAttribute attribut = (AbstractAttribute) it.next();
            buffer.append(attribut.getPName());
            buffer.append(":");
            buffer.append(mapAttributeName.get(attribut));
            buffers[i] = buffer.toString();
            i++;
        }
        return buffers;
    }

    public long getProceedTrack() {
        return tracker;
    }

    public void setTracker(String name) {
        strTracker = name;
    }

    public String getTracker() {
        return strTracker;
    }

    /**
     * Reset.
     * @see com.francetelecom.odis.component.stat.IStat#reset()
     */
    public void reset() {
        mapAttributeName.clear();
    }

    /**
     * Bind.
     * @see com.francetelecom.odis.component.admin.IPlugin#bind()
     */
    public void bind() {
        oldStat = StatProvider.getStatEngine();
        StatProvider.setStatEngine(this);
        JMX.getInstance().registerMBean(getName(), this);
    }

    /**
     * Gets the description.
     * @return the description
     * @see com.francetelecom.odis.component.admin.IPlugin#getDescription()
     */
    public String getDescription() {
        return null;
    }

    /**
     * Gets the name.
     * @return the name
     * @see com.francetelecom.odis.component.admin.IPlugin#getName()
     */
    public String getName() {
        return NAME;
    }

    /**
     * Unbind.
     * @see com.francetelecom.odis.component.admin.IPlugin#unBind()
     */
    public void unbind() {
        JMX.getInstance().unregisterMBean(getName(), this);
        StatProvider.setStatEngine(oldStat);
    }
}
