package org.ow2.odis.plugin;

import javax.xml.XMLConstants;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.ValidatorHandler;
import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.output.SAXOutputter;
import org.ow2.odis.admin.IPlugin;
import org.ow2.odis.admin.IValidateXML;
import org.ow2.odis.admin.Launch;
import org.xml.sax.SAXException;

/**
 * The Class validateXML.
 * @since JDK1.5
 */
public class validateXML implements IValidateXML, IPlugin {
    /** The old validate. */
    private IValidateXML oldValidate = null;

    /** The Constant name. */
    private static final String name = "validateXML";

    /**
     * Validate document.
     * @param doc the doc
     * @throws SAXException the SAX exception
     * @throws JDOMException the JDOM exception
     * @see com.francetelecom.odis.component.admin.IValidateXML#validateDocument(org.jdom.Document)
     */
    public void validateDocument(Document doc) throws SAXException, JDOMException {
        final String sl = XMLConstants.W3C_XML_SCHEMA_NS_URI;
        SchemaFactory factory = SchemaFactory.newInstance(sl);
        StreamSource ss = new StreamSource("etc/ODISXMLSchema3.xsd");
        Schema schema = factory.newSchema(ss);
        SAXParserFactory spf = SAXParserFactory.newInstance();
        spf.setNamespaceAware(true);
        ValidatorHandler vh = schema.newValidatorHandler();
        SAXOutputter so = new SAXOutputter(vh);
        so.output(doc);
    }

    /**
     * Bind the plugin.
     * @see com.francetelecom.odis.component.admin.IPlugin#add()
     */
    public void bind() {
        oldValidate = Launch.getInstance().getValidateXML();
        Launch.getInstance().setValidateXML(this);
    }

    /**
     * Unbind the plugin.
     * @see com.francetelecom.odis.component.admin.IPlugin#unBind()
     */
    public void unbind() {
        Launch.getInstance().setValidateXML(oldValidate);
    }

    /**
     * Gets the description.
     * @return the description
     * @see com.francetelecom.odis.component.admin.IPlugin#getDescription()
     */
    public String getDescription() {
        return null;
    }

    /**
     * Gets the name.
     * @return the name
     * @see com.francetelecom.odis.component.admin.IPlugin#getName()
     */
    public String getName() {
        return name;
    }
}
