package org.ow2.odis.proceed.admin;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * This command return the FullXML configuration
 * 
 * @author Arnaud Constancin _ France-Telecom R&D _ OrangeLabs
 * 
 */
public abstract class AbstractAdminCommand implements IAdminCommand {

	public static final String EOL = System.getProperty("line.separator");
	private static final Logger LOGGER = Monolog.initialize().getLogger(AbstractAdminCommand.class.getName());

	private String name = "NOT DEFINED";
	private String usage = "NOT DEFINED";

	/**
	 * 
	 */
	private AbstractAdminCommand() {

	}

	/**
	 * @param name
	 */
	public AbstractAdminCommand(String name) {
		super();
		this.name = name;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.francetelecom.odis.component.proceed.admin.IAdminCommand#run(java.lang.String)
	 */
	public abstract String run(String commandArgs) throws AdminCommandException;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.francetelecom.osgi.admin.commands.IAdminCommand#doesMatch(java.lang.String)
	 */
	public boolean matchesWithCommand(String commandName) {
		boolean isMatching = false;
		if (commandName == null || commandName.equals("NOT DEFINED")) {
			LOGGER.log(BasicLevel.ERROR, "This command is NULL or NOT DEFINED : " + this.getClass().getName());
		} else {
			isMatching = name.toLowerCase().equals(commandName.trim().toLowerCase());
			LOGGER.log(BasicLevel.DEBUG, "Test command matching : between " + getName() + " and " + commandName + " => isMatching = " + isMatching);
		}
		return isMatching;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.francetelecom.odis.component.proceed.admin.IAdminCommand#getName()
	 */
	public String getName() {
		return name;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.francetelecom.odis.component.proceed.admin.IAdminCommand#getUsage()
	 */
	public String getUsage() {
		return usage;
	}

	/**
	 * @param usage
	 */
	public void setUsage(String usage) {
		this.usage = usage;
	}

}
