package org.ow2.odis.proceed.admin;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.exception.OdisFactoryException;


/**
 * This default factory generates instance of class found in the class loader.
 * 
 * @author Arnaud Constancin
 */
public class AdminCommandFactory {

	/** logger of the class. */
	private static final Logger LOGGER = Monolog.initialize().getLogger(AdminCommandFactory.class.getName());

	/**
	 * This default factory generates instance of class found in the class path.
	 * 
	 * It
	 * 
	 * @see com.francetelecom.odis.component.loader.IFactory#getInstanceOf(java.lang.String)
	 */
	public Object getInstanceOf(String className) throws OdisFactoryException {
		Object result = null;

		try {
			Class classe = Class.forName(className);
			result = classe.newInstance();
		} catch (ClassNotFoundException e) {
			StringBuffer buff;
			buff = new StringBuffer("unable to found class \"");
			buff.append(className);
			buff.append("\" class");
			LOGGER.log(BasicLevel.ERROR, buff.toString());
			throw new OdisFactoryException(buff.toString());
		} catch (InstantiationException e) {
			StringBuffer buff;
			buff = new StringBuffer("unable to instanciate class \"");
			buff.append(className);
			buff.append("\" class");
			LOGGER.log(BasicLevel.ERROR, buff.toString());
			throw new OdisFactoryException(buff.toString());
		} catch (IllegalAccessException e) {
			StringBuffer buff = new StringBuffer("unable to instanciate class \"");
			buff.append(className);
			buff.append("\" class");
			LOGGER.log(BasicLevel.ERROR, buff.toString());
			throw new OdisFactoryException(buff.toString());
		} catch (ClassCastException e) {
			StringBuffer buff = new StringBuffer("try to use ");
			buff.append(className);
			buff.append(" class but it does not implement requierd interface");
			LOGGER.log(BasicLevel.ERROR, buff.toString());
			throw new OdisFactoryException(buff.toString());
		}
		return result;
	}
}
