package org.ow2.odis.proceed.admin.commands;

import org.ow2.odis.node.Node;
import org.ow2.odis.node.NodeManager;

import org.ow2.odis.proceed.admin.AbstractAdminCommand;
import org.ow2.odis.proceed.admin.AdminCommandException;

/**
 * This command return the state of different parts of a node
 * 
 * @author Arnaud Constancin _ France-Telecom R&D _ OrangeLabs
 * 
 */
public class GetNodeStatesCommand extends AbstractAdminCommand {

	/**
 	 */
	public GetNodeStatesCommand() {
		super("GetNodeStates");
		setUsage(getName() + " NodeName : return states of Node, EngineIn, EngineOut and Proceed");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.francetelecom.osgi.admin.commands.IAdminCommand#getCommandName()
	 */
	public String run(String commandArgs) throws AdminCommandException {
		StringBuffer sb = new StringBuffer();
		String nodeName = null;
		Node node = null;
		try {
			nodeName = commandArgs.trim();
			node = NodeManager.getInstance().getNode(nodeName);
		} catch (Exception e) {
			throw new AdminCommandException(e);
		}

		if (null == node) {
			throw new AdminCommandException("Node \"" + nodeName + "\" Not Found");
		} else {
			try {
				sb.append("<node name=\"" + nodeName + "\" state =\"" + node.getNodeState().getStateName() + "\">" + EOL);
				sb.append("  <engine_in state=\"" + node.getStateEngineIn() + "\"/>" + EOL);
				sb.append("  <engine_out state=\"" + node.getStateEngineOut() + "\"/>" + EOL);
				sb.append("  <engine_proceed state=\"" + node.getStateEngineProceed() + "\"/>" + EOL);
				sb.append("</node>" + EOL);
			} catch (Exception e) {
				throw new AdminCommandException(e);
			}
		}
		return sb.toString();
	}
}
