package org.ow2.odis.proceed.admin.commands;

import org.ow2.odis.node.Node;
import org.ow2.odis.node.NodeManager;

import org.ow2.odis.proceed.admin.AbstractAdminCommand;
import org.ow2.odis.proceed.admin.AdminCommandException;

/**
 * This command return the state of different parts of a node
 * 
 * @author Arnaud Constancin _ France-Telecom R&D _ OrangeLabs
 * 
 */
public class SetNodeStateCommand extends AbstractAdminCommand {

	/**
 	 */
	public SetNodeStateCommand() {
		super("SetNodeState");
		setUsage(getName() + " NodeName X : set the state X of the Node. where X in O..5 for 0=LOADED 1=INITIALIZED 2=START 3=SUSPEND 4=STOP 5=UNLOADED");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.francetelecom.osgi.admin.commands.IAdminCommand#getCommandName()
	 */
	public String run(String commandArgs) throws AdminCommandException {
		StringBuffer sb = new StringBuffer();
		String nodeName = null;
		int stateNum = -1;
		Node node = null;
		boolean setStateIsOK = false;
		try {
			nodeName = commandArgs.substring(0, commandArgs.indexOf(" ")).trim();
			stateNum = Integer.parseInt(commandArgs.substring(commandArgs.indexOf(" ")).trim());
			node = NodeManager.getInstance().getNode(nodeName);
		} catch (Exception e) {
			throw new AdminCommandException(e);
		}

		if (null == node) {
			throw new AdminCommandException("Node \"" + nodeName + "\" Not Found");
		} else {
			try {
				setStateIsOK = node.setState(stateNum);
			} catch (Exception e) {
				throw new AdminCommandException(e);
			}

			if (setStateIsOK) {
				sb.append("<set_state node=\"" + nodeName + "\" new_state=\"" + stateNum + "\">" + EOL);
				sb.append("<message>State of node  " + nodeName + " has been set to " + stateNum + "</message>" + EOL);
				sb.append("</set_state>" + EOL);
			} else {
				String errorMessage = "State of node  " + nodeName + " has not changed !";
				// sb.append(errorMessage + EOL);
				throw new AdminCommandException(errorMessage);
			}
		}

		return sb.toString();
	}

}
