package org.ow2.odis.proceed.console;

import java.awt.BorderLayout;
import javax.swing.DefaultListSelectionModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumnModel;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * The Class AbstractConsole.
 */
public abstract class AbstractConsole extends JDialog {
    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize().getLogger(AbstractConsole.class.getName());

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** The j content pane. */
    private transient JPanel jContentPane = null;

    /** The j scroll pane. */
    private transient JScrollPane jScrollPane = null;

    /** The j table. */
    private transient JTable jTable = null;

    /** The j panel entete. */
    private transient JPanel jPanelEntete = null;

    /** The j label description. */
    private transient JLabel jLabelDescription = null;

    /** The j text field. */
    private transient JTextField jTextField = null;

    /** The table command model. */
    private transient ConsoleTableModel tableCommandModel = null; // @jve:decl-index=0:visual-constraint=""

    /** The j button purge. */
    private transient JButton jButtonPurge = null;

    /** The j button save. */
    private transient JButton jButtonSave = null;

    /** The j button stop. */
    private transient JButton jButtonStop = null;

    /** The j split pane. */
    private transient JSplitPane jSplitPane = null;

    /**
     * Save model.
     */
    protected abstract void saveModel();

    /**
     * Reset table.
     */
    protected abstract void resetTable();

    /**
     * Stop acquiring.
     */
    protected abstract void stopAcquiring();

    /**
     * Change selection.
     */
    abstract protected void changeSelection();

    /**
     * This method initializes jScrollPane.
     * @return javax.swing.JScrollPane
     */
    private JScrollPane getJScrollPane() {
        if (jScrollPane == null) {
            jScrollPane = new JScrollPane();
            jScrollPane.setViewportView(getJTable());
        }
        return jScrollPane;
    }

    /**
     * This method initializes jTable.
     * @return javax.swing.JTable
     */
    protected JTable getJTable() {
        if (jTable == null) {
            jTable = new JTable();
            jTable.setShowGrid(true);
            jTable.setModel(getTableCommandModel());
            final DefaultListSelectionModel selectionModel = new DefaultListSelectionModel();
            jTable.setSelectionModel(selectionModel);
            jTable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_LAST_COLUMN);
            selectionModel.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
            selectionModel.addListSelectionListener(new SelectionListener());
        }
        return jTable;
    }

    /**
     * The listener interface for receiving selection events. The class that is interested in processing a selection event implements this
     * interface, and the object created with that class is registered with a component using the component's
     * <code>addSelectionListener<code> method. When
     * the selection event occurs, that object's appropriate
     * method is invoked.
     *
     * @see SelectionEvent
     */
    class SelectionListener implements ListSelectionListener {
        /**
         * @param evt
         * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
         */
        public void valueChanged(ListSelectionEvent evt) {
            changeSelection();
        }
    }

    /**
     * This method initializes jPanel.
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel() {
        if (jPanelEntete == null) {
            jLabelDescription = new JLabel();
            jLabelDescription.setText("Description");
            jPanelEntete = new JPanel();
            jPanelEntete.add(jLabelDescription, null);
            jPanelEntete.add(getJTextField(), null);
            jPanelEntete.add(getJButtonPurge(), null);
            jPanelEntete.add(getJButtonSave(), null);
            jPanelEntete.add(getJButtonStop(), null);
        }
        return jPanelEntete;
    }

    /**
     * This method initializes jTextField.
     * @return javax.swing.JTextField
     */
    protected JTextField getJTextField() {
        if (jTextField == null) {
            jTextField = new JTextField();
            jTextField.setPreferredSize(new java.awt.Dimension(140, 20));
            jTextField.setEditable(false);
        }
        return jTextField;
    }

    /**
     * This method initializes tableCommandModel.
     * @return TableCommandModel
     */
    protected ConsoleTableModel getTableCommandModel() {
        if (tableCommandModel == null) {
            tableCommandModel = new ConsoleTableModel();
        }
        return tableCommandModel;
    }

    /**
     * This method initializes jButtonPurge.
     * @return javax.swing.JButton
     */
    protected JButton getJButtonPurge() {
        if (jButtonPurge == null) {
            jButtonPurge = new JButton();
            jButtonPurge.setText("Purge");
            ImageIcon icon = null;
            String ressource = "/org/ow2/odis/proceed/console/corbeille.gif";
            try {
                icon = new ImageIcon(getClass().getResource(ressource));
                jButtonPurge.setIcon(icon);
            } catch (RuntimeException e1) {
                if (LOGGER.isLoggable(BasicLevel.WARN)) {
                    LOGGER.log(BasicLevel.WARN, "unable to find ressource :" + ressource);
                }
            }
            ressource = "/org/ow2/odis/proceed/console/corbeille_over.gif";
            try {
                icon = new ImageIcon(getClass().getResource(ressource));
                jButtonPurge.setRolloverIcon(icon);
            } catch (RuntimeException e1) {
                if (LOGGER.isLoggable(BasicLevel.WARN)) {
                    LOGGER.log(BasicLevel.WARN, "unable to find ressource :" + ressource);
                }
            }
            jButtonPurge.setToolTipText("Clear the log and set the T0 used by delta time computing");
            jButtonPurge.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent event) {
                    resetTable();
                }
            });
        }
        return jButtonPurge;
    }

    /**
     * This method initializes jButtonSave.
     * @return javax.swing.JButton
     */
    protected JButton getJButtonSave() {
        if (jButtonSave == null) {
            jButtonSave = new JButton();
            jButtonSave.setText("Save");
            ImageIcon icon = null;
            String ressource = "/org/ow2/odis/proceed/console/save.gif";
            try {
                icon = new ImageIcon(getClass().getResource(ressource));
                jButtonSave.setIcon(icon);
            } catch (RuntimeException e1) {
                if (LOGGER.isLoggable(BasicLevel.WARN)) {
                    LOGGER.log(BasicLevel.WARN, "unable to find ressource :" + ressource);
                }
            }
            ressource = "/org/ow2/odis/proceed/console/save_over.gif";
            try {
                icon = new ImageIcon(getClass().getResource(ressource));
                jButtonSave.setRolloverIcon(icon);
            } catch (RuntimeException e1) {
                if (LOGGER.isLoggable(BasicLevel.WARN)) {
                    LOGGER.log(BasicLevel.WARN, "unable to find ressource :" + ressource);
                }
            }
            jButtonSave.setToolTipText("save the log to file");
            jButtonSave.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    saveModel();
                }
            });
        }
        return jButtonSave;
    }

    /**
     * This is the default constructor.
     */
    public AbstractConsole() {
        super();
        initialize();
    }

    /**
     * This method initializes this system.
     */
    private void initialize() {
        this.setTitle("Console Event");
        this.setBounds(new java.awt.Rectangle(0, 0, 947, 400));
        this.setContentPane(getJContentPane());
        final TableColumnModel colMdl = getJTable().getColumnModel();
        colMdl.getColumn(0).setPreferredWidth(150);
        colMdl.getColumn(1).setPreferredWidth(100);
        colMdl.getColumn(2).setPreferredWidth(50);
        colMdl.getColumn(3).setPreferredWidth(250);
        getJTable().setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_LAST_COLUMN);
    }

    /**
     * This method initializes jContentPane.
     * @return javax.swing.JPanel
     */
    private JPanel getJContentPane() {
        if (jContentPane == null) {
            jContentPane = new JPanel();
            jContentPane.setLayout(new BorderLayout());
            jContentPane.add(getJSplitPane(), java.awt.BorderLayout.CENTER);
            jContentPane.add(getJPanel(), java.awt.BorderLayout.NORTH);
        }
        return jContentPane;
    }

    /**
     * This method initializes jButtonPause.
     * @return javax.swing.JButton
     */
    protected JButton getJButtonStop() {
        if (jButtonStop == null) {
            jButtonStop = new JButton();
            jButtonStop.setText("Stop");
            ImageIcon icon = null;
            String ressource = "/org/ow2/odis/proceed/console/stop.gif";
            try {
                icon = new ImageIcon(getClass().getResource(ressource));
                jButtonStop.setIcon(icon);
            } catch (RuntimeException e1) {
                if (LOGGER.isLoggable(BasicLevel.WARN)) {
                    LOGGER.log(BasicLevel.WARN, "unable to find ressource :" + ressource);
                }
            }
            ressource = "/org/ow2/odis/proceed/console/stop_over.gif";
            try {
                icon = new ImageIcon(getClass().getResource(ressource));
                jButtonStop.setRolloverIcon(icon);
            } catch (RuntimeException e1) {
                if (LOGGER.isLoggable(BasicLevel.WARN)) {
                    LOGGER.log(BasicLevel.WARN, "unable to find ressource :" + ressource);
                }
            }
            jButtonStop.setToolTipText("stop acquiring");
            jButtonStop.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    stopAcquiring();
                }
            });
        }
        return jButtonStop;
    }

    /**
     * This method initializes jSplitPane.
     * @return javax.swing.JSplitPane
     */
    protected JSplitPane getJSplitPane() {
        if (jSplitPane == null) {
            jSplitPane = new JSplitPane();
            jSplitPane.setLeftComponent(getJScrollPane());
        }
        return jSplitPane;
    }
} // @jve:decl-index=0:visual-constraint="10,10"
