package org.ow2.odis.proceed.console;

import java.util.Vector;
import javax.swing.table.AbstractTableModel;

/**
 * The Class ConsoleTableModel.
 */
public class ConsoleTableModel extends AbstractTableModel {
    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;

    private static final int DEFAULT_SIZE_MAX = 1000;

    /** size max of the table if the size = -1 there is no limit. */
    private int sizeMaxRaw = DEFAULT_SIZE_MAX;

    /** The data. */
    private Vector data = new Vector();

    /** The Constant header. */
    private static final String[] HEADERS = { "from", "created", "delta", "command" };

    /**
     * Gets the row count.
     * @return the row count
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount() {
        return data.size();
    }

    /**
     * Gets the column count.
     * @return the column count
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount() {
        return HEADERS.length;
    }

    /**
     * Gets the column name.
     * @param columnIndex the column index
     * @return the column name
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    public String getColumnName(int columnIndex) {
        String result = "";
        if (columnIndex < HEADERS.length) {
            result = HEADERS[columnIndex];
        }
        return result;
    }

    /**
     * Gets the column class.
     * @param columnIndex the column index
     * @return the column class
     * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
     */
    public Class getColumnClass(int columnIndex) {
        return HEADERS[columnIndex].getClass();
    }

    /**
     * Checks if is cell editable.
     * @param rowIndex the row index
     * @param columnIndex the column index
     * @return true, if is cell editable
     * @see javax.swing.table.AbstractTableModel#isCellEditable(int, int)
     */
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return false;
    }

    /**
     * Gets the value at.
     * @param rowIndex the row index
     * @param columnIndex the column index
     * @return the value at
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt(int rowIndex, int columnIndex) {
        Object result = null;
        try {
            Object[] row = (Object[]) data.get(rowIndex);
            if (row.length > columnIndex) {
                result = row[columnIndex];
            }
        } catch (RuntimeException e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * Sets the value at.
     * @param aValue the a value
     * @param rowIndex the row index
     * @param columnIndex the column index
     * @see javax.swing.table.AbstractTableModel#setValueAt(java.lang.Object, int, int)
     */
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        Object[] row = (Object[]) data.get(rowIndex);
        Object oldValue = row[columnIndex];
        row[columnIndex] = aValue;
        if (!oldValue.equals(aValue)) {
            fireTableCellUpdated(rowIndex, columnIndex);
        }
    }

    /**
     * aim : <br>
     * verify if the size max is reached <br>
     * add the new raw to the table.
     * @param raw the raw
     */
    public void addValue(Object[] raw) {
        // test if the size max is reached and in this case remove the oldest
        // element
        if (sizeMaxRaw > 0 && data.size() > sizeMaxRaw) {
            data.remove(data.size() - 1);
            fireTableRowsDeleted(data.size() - 1, data.size() - 1);
        }
        data.add(0, raw);
        fireTableRowsInserted(0, 0);
    }

    /**
     * clear all the table value and inform the GUI.
     */
    public void clearValue() {
        data = new Vector();
        fireTableDataChanged();
    }

    /**
     * Gets the size max raw.
     * @return the size max raw
     */
    public int getSizeMaxRaw() {
        return sizeMaxRaw;
    }

    /**
     * Sets the size max raw.
     * @param sizeMax (int) fix the max raw of the table if the size = -1 there is no limit
     */
    public void setSizeMaxRaw(int sizeMax) {
        this.sizeMaxRaw = sizeMax;
    }
}
