package org.ow2.odis.proceed.injector;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;
import javax.swing.border.SoftBevelBorder;

/**
 * AbstractInjector defines the GUI for the injector. The actual actions taken are to be defined by the implementing class.
 * @author olivier beyler
 */
abstract class AbstractContextualInjector extends AbstractInjector {
    
	
    
    public AbstractContextualInjector() {
        super();
    }

	protected JPanel getSouthPanel() {
		JPanel panel = super.getSouthPanel();
		panel.add(getContextPane1());
        return panel;
	}
	
	/**
     * Initializes ctrlPane1 if needed.
     * @return The panel (with all controls inside).
     */
    private JPanel getContextPane1() {
        if (contextPane1 == null) {
            contextPane1 = new JPanel();
            BoxLayout contextLayout = new BoxLayout(contextPane1, BoxLayout.PAGE_AXIS);
            contextPane1.setBorder(new SoftBevelBorder(BevelBorder.RAISED, null, null, null, null));
            contextPane1.setLayout(contextLayout);
            contextPane1.add(getUseContextPane());
            contextPane1.add(getIdentityPanel());
            contextPane1.add(getSubIdentityPanel());
            contextPane1.add(getDetailPanel());
        }
        return contextPane1;
    }
    
    /**
     * Initializes the context panel if needed.
     * @return The format panel (with ad hoc controls inside).
     */
    private JPanel getUseContextPane() {
        if (contextPane == null) {
            contextPane = new JPanel();
            FlowLayout layout = new FlowLayout(FlowLayout.LEADING);
            contextPane.setLayout(layout);
            // Checkbox - use context or not
            getUseContextCheckbox().setSelected(false);
            contextPane.add(Box.createHorizontalGlue());
            contextPane.add(Box.createRigidArea(new Dimension(1, 0)));
            contextPane.add(getUseContextCheckbox());
        }
        return contextPane;
    }
    
    /**
     * Initializes the format panel if needed.
     * @return The format panel (with ad hoc controls inside).
     */
    private JPanel getIdentityPanel() {
        if (identityPanel == null) {
        	identityPanel = new JPanel();
            FlowLayout layout = new FlowLayout(FlowLayout.LEADING);
            identityPanel.setLayout(layout);
            if (identityLabel == null) identityLabel = new JLabel("Identity");
            identityLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
            identityLabel.setFont(new java.awt.Font(FONT_NAME, Font.PLAIN, FONT_SIZE));
            identityPanel.add(identityLabel);
            identityLabel.setPreferredSize(new java.awt.Dimension(75, 15));
            identityPanel.add(getIdentityField());
            identityPanel.add(Box.createHorizontalGlue());
            identityPanel.add(Box.createRigidArea(new Dimension(1, 0)));
        }
        return identityPanel;
    }
    
    /**
     * Initializes the format panel if needed.
     * @return The format panel (with ad hoc controls inside).
     */
    private JPanel getSubIdentityPanel() {
        if (subIdentityPanel == null) {
        	subIdentityPanel = new JPanel();
            FlowLayout layout = new FlowLayout(FlowLayout.LEADING);
            subIdentityPanel.setLayout(layout);
            if (subIdentityLabel == null) subIdentityLabel = new JLabel("Sub-identity");
            subIdentityLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
            subIdentityLabel.setFont(new java.awt.Font(FONT_NAME, Font.PLAIN, FONT_SIZE));
            subIdentityPanel.add(subIdentityLabel);
            subIdentityLabel.setPreferredSize(new java.awt.Dimension(74, 15));
            subIdentityPanel.add(getSubidentityField());
            subIdentityPanel.add(Box.createHorizontalGlue());
            subIdentityPanel.add(Box.createRigidArea(new Dimension(1, 0)));
        }
        return subIdentityPanel;
    }
    
    /**
     * Initializes the detail panel if needed.
     * @return The detail panel.
     */
    private JPanel getDetailPanel() {
        if (detailPanel == null) {
        	detailPanel = new JPanel();
            FlowLayout layout = new FlowLayout(FlowLayout.LEADING);
            detailPanel.setLayout(layout);
            if (detailLabel == null) detailLabel = new JLabel("Detail");
            detailLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
            detailLabel.setFont(new java.awt.Font(FONT_NAME, Font.PLAIN, FONT_SIZE));
            detailPanel.add(detailLabel);
            detailLabel.setPreferredSize(new java.awt.Dimension(74, 15));
            detailPanel.setPreferredSize(new java.awt.Dimension(74, 15));
            detailPanel.add(getDetailField());
            detailPanel.add(Box.createHorizontalGlue());
            detailPanel.add(Box.createRigidArea(new Dimension(1, 0)));
        }
        return detailPanel;
    }
    /**
     * Initializes the "use context" checkbox if needed.
     * @return The "use context" checkbox.
     */
    private JCheckBox getUseContextCheckbox() {
        if (useContextCheckbox == null) {
            useContextCheckbox = new JCheckBox("Use context");
            useContextCheckbox.setFont(boldFont);
        }
        return useContextCheckbox;
    }
    /**
     * Tells whether "use context" is checked.
     * @return true if checked.
     */
    public boolean useContext() {
        return getUseContextCheckbox().isSelected();
    }

    /**
     * Returns the current context identity.
     * @return a String holding the identity.
     */
    public String getIdentity() {
        return getIdentityField().getText();
    }

    /**
     * Returns the current context detail.
     * @return a String holding the detail.
     */
    public String getDetail() {
        return getDetailField().getText();
    }
    /**
     * Returns the current context sub-identity.
     * @return a String holding the sub-identity.
     */
    public String getSubidentity() {
        return getSubidentityField().getText();
    }

    /**
     * Initializes the "identity" field if needed.
     * @return The "identity" field.
     */
    private JTextField getIdentityField() {
        if (identityField == null) {
            identityField = new JTextField("", 10);
        }
        return identityField;
    }

    /**
     * Initializes the "subidentity" field if needed.
     * @return The "subidentity" field.
     */
    private JTextField getSubidentityField() {
        if (subidentityField == null) {
            subidentityField = new JTextField("", 10);
        }
        return subidentityField;
    }
    
    /**
     * Initializes the "subidentity" field if needed.
     * @return The "subidentity" field.
     */
    private JTextField getDetailField() {
        if (detailField == null) {
        	detailField = new JTextField("", 20);
        }
        return detailField;
    }        
	
    /**
	 * This constructor is used by the getGUIBuilderInstance method to
	 * provide an instance of this class which has not had it's GUI elements
	 * initialized (ie, initGUI is not called in this constructor).
	 */
	public AbstractContextualInjector(Boolean initGUI) {
		super();
	}
}
