package org.ow2.odis.proceed.injector;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.UIManager;
import javax.swing.border.BevelBorder;
import javax.swing.border.SoftBevelBorder;
/**
 * AbstractInjector defines the GUI for the injector. The actual actions taken are to be defined by the implementing class.
 * @author mobo7205
 */
abstract class AbstractInjector extends JDialog {
	
	protected final String FONT_NAME = UIManager.getFont("Label.font").getFamily();
	
	protected final int FONT_SIZE = 12;
	
	protected final Font plainFont = new Font(FONT_NAME, Font.PLAIN, FONT_SIZE);
    
	protected final Font boldFont = new Font(FONT_NAME, Font.BOLD, FONT_SIZE);

	/** The "use context" checkbox. If selected, events are sent as routing objects with the specified context. */
	protected JCheckBox useContextCheckbox = null;

    /** The "identity" field. Defines the main identity for the context. */
    protected JTextField identityField = null;

    /** The "subidentity" field. Defines the sub-udentity for the context. */
    protected JTextField subidentityField = null;
    
    /** The "detail" field. Defines the detail for the context. */
    protected JTextField detailField = null;
    
	/** Sub-panel for controls layout */
    protected JPanel contextPane1 = null;
    
    /** The "identity" panel.. */
    protected JPanel identityPanel = null;

    /** The "subIdentity" panel.. */
    protected JPanel subIdentityPanel = null;

    /** The "detail" panel.. */
    protected JPanel detailPanel = null;
	
    /** The "context" panel. Holds the controls deciding if context is used. */
    protected JPanel contextPane = null;

    /** The identity label. To show the number of events sent. */
    protected JLabel identityLabel = null;

    /** The subidentity label. To show the number of events sent. */
    protected JLabel subIdentityLabel = null;

    /** The detail label. To show the number of events sent. */
    protected JLabel detailLabel = null;
	
	/** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** Action command constant */
    private static final String CMD_FULL = "full";

    /** Action command constant */
    private static final String CMD_BY_LINE = "byLine";

    /** Action command constant */
    private static final String CMD_SPLIT = "split";

    /** Action command constant */
    private static final String CMD_ONCE = "once";

    /** Action command constant */
    private static final String CMD_CYCLE = "cycle";

    /** Our standard margin */
    private static final int MARGIN = 10;

    /** Our standard width */
    private static final int WIDTH = 900;

    /** Our standard height */
    private static final int HEIGHT = 400;

    /** Default period for automated event generation */
    private static final int PERIOD = 1000;

    /* GUI components */
    /** The content pane. */
    private JPanel jContentPane = null;

    /** The scroll pane for the text area. */
    private JScrollPane jScrollPane = null;

    /** Panel holding all the controls. */
    private JPanel southPanel = null;

    /** Sub-panel for controls layout */
    private JPanel controlsPane1 = null;

    /** Sub-panel for action layout */
    private JPanel actionPane1 = null;

    /** Sub-panel for controls layout */
    private JPanel ctrlPane2 = null;

    /** The "identity" panel.. */
    private JPanel formatPane = null;

    /** The "timing" panel. Holds the controls deciding how events are sent. */
    private JPanel timingPane = null;

    /** The "title" panel. */
    private JPanel titlePanel = null;

    /** The "count" panel. Displays the number of events sent. */
    private JPanel countPane = null;

    /** The text area. Holds the contents for the events to be generated. */
    private JTextArea jTextArea = null;

    /** The "event" button. Triggers event generation. */
    private JButton eventBtn = null;

    /** The "load" button. Initializes the text area from a file. */
    private JButton loadBtn = null;

    /** The "reset" button. Empties the text area. */
    private JButton resetBtn = null;

    /** The "full" button. If selected, the whole text is sent as one event. */
    private JToggleButton fullButton = null;

    /** The "by line" button. If selected, each line is sent as one event. */
    private JToggleButton byLineButton = null;

    /** The "split" button. If selected, the text is split according to separator and sent as that many events. */
    private JToggleButton splitButton = null;

    /** The "once" button. If selected, events are sent only on user action. */
    private JToggleButton onceButton = null;

    /** The "cycle" button. If selected, events are sent automatically based on the period. */
    private JToggleButton cycleButton = null;

    /** The "separator" field. Holds the separator used with the "split" button. */
    private JTextField separatorField = null;

    /** The "period" field. Defines the period (ms) for automated event generation. */
    private JTextField periodField = null;
    
    /** The "ms" label. */
    private JLabel msLabel = new JLabel("ms");

    /** The count label. To show the number of events sent. */
    private JLabel countLabel = new JLabel("0");

    /** The title label. */
    private JLabel titleLabel = new JLabel("Injector configuration");
    private JSeparator jSeparator1;

    /* Constructor and related stuff. */
    /** Instantiates a new abstract injector. */
    protected AbstractInjector() {
        super();
        initialize();
    }
    
    /** Actually initializes the GUI. */
    private void initialize() {
    	this.setBounds(0, 0, WIDTH, HEIGHT);
        this.setContentPane(getJContentPane());
    }

    /* Private accessors/initializers. */
    /**
     * Initializes the content pane if needed.
     * @return The content pane
     */
    private JPanel getJContentPane() {
        if (jContentPane == null) {
            jContentPane = new JPanel();
            jContentPane.setLayout(new BorderLayout());
            jContentPane.add(getJScrollPane(), BorderLayout.CENTER);
            jContentPane.add(getCtrlPane2(), BorderLayout.EAST);
            jContentPane.add(getSouthPanel(), BorderLayout.SOUTH);
        }
        return jContentPane;
    }

    /**
     * Initializes the scroll pane if needed.
     * @return The scroll pane (linked to the text area).
     */
    private JScrollPane getJScrollPane() {
        if (jScrollPane == null) {
            jScrollPane = new JScrollPane();
            jScrollPane.setViewportView(getJTextArea());
        }
        return jScrollPane;
    }

    /**
     * Initializes the text area if needed.
     * @return The text area.
     */
    private JTextArea getJTextArea() {
        if (jTextArea == null) {
            jTextArea = new JTextArea();
        }
        return jTextArea;
    }
   
    /**
     * Initializes southPanel if needed.
     * @return The panel (with all controls inside).
     */
    protected JPanel getSouthPanel() {
        if (southPanel == null) {
        	southPanel = new JPanel();        	
        	BoxLayout layout = new BoxLayout(southPanel, BoxLayout.LINE_AXIS);
            southPanel.setLayout(layout);
            southPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
            southPanel.setPreferredSize(new java.awt.Dimension(882, 170));
            southPanel.add(getControlePanel());
        }
        return southPanel;
    }

    
    /**
     * Initializes controlPane1 if needed.
     * @return The panel (with all controls inside).
     */
    private JPanel getControlePanel() {
        if (controlsPane1 == null) {
        	controlsPane1 = new JPanel();
            BoxLayout controleLayout = new BoxLayout(controlsPane1, BoxLayout.PAGE_AXIS);
            controlsPane1.setLayout(controleLayout);
            controlsPane1.setBorder(new SoftBevelBorder(BevelBorder.RAISED, null, null, null, null));
            
            controlsPane1.add(getTitlePanel());
            controlsPane1.add(getFormatPane());
            controlsPane1.add(getTimingPane());
            controlsPane1.add(getEventBtn());
        }
        return controlsPane1;
    }

    
    /**
     * This method initializes ctrlPane2.
     * @return javax.swing.JPanel
     */
    private JPanel getCtrlPane2() {
        if (ctrlPane2 == null) {
            ctrlPane2 = new JPanel();
            BoxLayout layout = new BoxLayout(ctrlPane2, BoxLayout.PAGE_AXIS);
            ctrlPane2.setLayout(layout);
            ctrlPane2.add(Box.createRigidArea(new Dimension(0, MARGIN)));
            ctrlPane2.add(getLoadBtn(), null);
            getLoadBtn().setAlignmentX(Component.CENTER_ALIGNMENT);
            ctrlPane2.add(Box.createRigidArea(new Dimension(0, MARGIN)));
            ctrlPane2.add(getResetBtn(), null);
            getResetBtn().setAlignmentX(Component.CENTER_ALIGNMENT);
            // Write the labels
            JLabel lab = new JLabel("SENT");
            lab.setFont(boldFont);
            lab.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
            lab.setAlignmentX(Component.CENTER_ALIGNMENT);
            ctrlPane2.add(lab);
            ctrlPane2.setAlignmentX(Component.CENTER_ALIGNMENT);
            countLabel.setFont(plainFont);
            ctrlPane2.add(countLabel);
        }
        return ctrlPane2;
    }

    /**
     * Initializes the format panel if needed.
     * @return The format panel (with ad hoc controls inside).
     */
    private JPanel getFormatPane() {
        if (formatPane == null) {
        	formatPane = new JPanel();
            FlowLayout layout = new FlowLayout(FlowLayout.LEADING);
            formatPane.setLayout(layout);
            // Build the button group
            ButtonGroup group = new ButtonGroup();
            group.add(getFullButton());
            group.add(getByLineButton());
            group.add(getSplitButton());
            // Default value
            getFullButton().setSelected(true);
            // Add buttons to panel
            formatPane.add(getFullButton());
            formatPane.add(getByLineButton());
            formatPane.add(getSplitButton());
            // Add separator field
            formatPane.add(getSeparatorField());
            formatPane.add(Box.createHorizontalGlue());
            formatPane.add(Box.createRigidArea(new Dimension(1, 0)));
        }
        return formatPane;
    }
    
    
    /**
     * Initializes the timing panel if needed.
     * @return The timing panel (with ad hoc controls inside).
     */
    private JPanel getTimingPane() {
        if (timingPane == null) {
            timingPane = new JPanel();
            FlowLayout layout = new FlowLayout(FlowLayout.LEADING);
            timingPane.setLayout(layout);
            // Build the button group
            ButtonGroup group = new ButtonGroup();
            group.add(getOnceButton());
            group.add(getCycleButton());
            // Default value
            getOnceButton().setSelected(true);
            // Add buttons to panel
            timingPane.add(getOnceButton());
            timingPane.add(getCycleButton());
            // Period field and label
            getPeriodField().setEnabled(false);
            timingPane.add(getPeriodField());
            msLabel.setEnabled(false);
            msLabel.setFont(plainFont);
            timingPane.add(msLabel);
            // Enabling/disabling period field
            getCycleButton().addChangeListener(new javax.swing.event.ChangeListener() {
                public void stateChanged(javax.swing.event.ChangeEvent e) {
                    getPeriodField().setEnabled(isCycle());
                    msLabel.setEnabled(isCycle());
                }
            });
        }
        return timingPane;
    }

    /**
     * Initializes the context panel if needed.
     * @return The format panel (with ad hoc controls inside).
     */
    private JPanel getTitlePanel() {
        if (titlePanel == null) {
        	titlePanel = new JPanel();
            FlowLayout layout = new FlowLayout(FlowLayout.LEADING);
            titlePanel.setLayout(layout);
            titleLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
            titlePanel.add(titleLabel);
            titleLabel.setFont(boldFont);
            titleLabel.setText("Injector configuration");
            titlePanel.add(Box.createHorizontalGlue());
            titlePanel.add(Box.createRigidArea(new Dimension(1, 0)));
            titlePanel.add(getJSeparator1());
        }
        return titlePanel;
    }

    /**
     * This method initializes eventBtn.
     * @return javax.swing.JButton
     */
    private JButton getEventBtn() {
        if (eventBtn == null) {
            eventBtn = new JButton("Inject");
            eventBtn.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    onEventBtn();
                }
            });
        }
        return eventBtn;
    }

    /**
     * This method initializes loadBtn
     * @return javax.swing.JButton
     */
    private JButton getLoadBtn() {
        if (loadBtn == null) {
            loadBtn = new JButton();
            loadBtn.setText("load");
            loadBtn.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    onLoadBtn();
                }
            });
        }
        return loadBtn;
    }

    /**
     * This method initializes loadBtn
     * @return javax.swing.JButton
     */
    private JButton getResetBtn() {
        if (resetBtn == null) {
            resetBtn = new JButton();
            resetBtn.setText("reset");
            resetBtn.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    onResetBtn();
                }
            });
        }
        return resetBtn;
    }

    /**
     * This method initializes periodField.
     * @return javax.swing.JTextField
     */
    protected JTextField getPeriodField() {
        if (periodField == null) {
            periodField = new JTextField(Integer.toString(PERIOD), 8);
        }
        return periodField;
    }

    /**
     * Initializes the "full" button if needed.
     * @return The "full" button.
     */
    private JToggleButton getFullButton() {
        if (fullButton == null) {
            fullButton = new JRadioButton("as one event");
            fullButton.setActionCommand(CMD_FULL);
            fullButton.setFont(plainFont);
        }
        return fullButton;
    }

    /**
     * Initializes the "by line" button if needed.
     * @return The "by line" button.
     */
    private JToggleButton getByLineButton() {
        if (byLineButton == null) {
            byLineButton = new JRadioButton("line by line");
            byLineButton.setActionCommand(CMD_BY_LINE);
            byLineButton.setFont(plainFont);
        }
        return byLineButton;
    }

    /**
     * Initializes the "split" button if needed.
     * @return The "split" button.
     */
    private JToggleButton getSplitButton() {
        if (splitButton == null) {
            splitButton = new JRadioButton("split with separator");
            splitButton.setActionCommand(CMD_SPLIT);
            splitButton.setFont(plainFont);
        }
        return splitButton;
    }

    /**
     * Initializes the "by line" button if needed.
     * @return The "by line" button.
     */
    private JToggleButton getOnceButton() {
        if (onceButton == null) {
            onceButton = new JRadioButton("at once");
            onceButton.setActionCommand(CMD_ONCE);
            onceButton.setFont(plainFont);
        }
        return onceButton;
    }

    /**
     * Initializes the "split" button if needed.
     * @return The "split" button.
     */
    private JToggleButton getCycleButton() {
        if (cycleButton == null) {
            cycleButton = new JRadioButton("automatically every");
            cycleButton.setActionCommand(CMD_CYCLE);
            cycleButton.setFont(plainFont);
        }
        return cycleButton;
    }

    /**
     * Initializes the "separator" field if needed.
     * @return The "separator" field.
     */
    private JTextField getSeparatorField() {
        if (separatorField == null) {
            separatorField = new JTextField("/", 2);
        }
        return separatorField;
    }

    /* Public accessors. */
    /**
     * Tells whether fullButton is selected.
     * @return true if selected.
     */
    public boolean isFull() {
        return getFullButton().isSelected();
    }

    /**
     * Tells whether byLineButton is selected.
     * @return true if selected.
     */
    public boolean isByLine() {
        return getByLineButton().isSelected();
    }

    /**
     * Tells whether byLineButton is selected.
     * @return true if selected.
     */
    public boolean isSplit() {
        return getSplitButton().isSelected();
    }

    /**
     * Tells whether cycleCheckBox is checked.
     * @return true if checked
     */
    public boolean isCycle() {
        return getCycleButton().isSelected();
    }

    /**
     * Returns the int value of the period, or 1000 if not a valid int value.
     * @return the period.
     */
    public int getPeriod() {
        int i = 0;
        if (periodField != null) {
            try {
                i = Integer.parseInt(periodField.getText());
            } catch (NumberFormatException e) {
                i = PERIOD;
                periodField.setText(Integer.toString(PERIOD));
            }
        }
        return i;
    }

    /**
     * Returns the current separator.
     * @return a String holding the separator.
     */
    public String getSeparator() {
        return getSeparatorField().getText();
    }

    /**
     * Returns the text currently used for event generation (actually the text in the text area).
     * @return the current text
     */
    public String getEventText() {
        return getJTextArea().getText();
    }

    /**
     * Sets the text used for event generation (actually the text in the text area).
     * @param text the text to set
     */
    public void setEventText(String text) {
        getJTextArea().setText(text);
    }

    /** For event sending notification */
    public void eventSent() {
        countLabel.setText("" + (Integer.parseInt(countLabel.getText()) + 1));
    }

    /* Abstract methods for GUI event handling. */
    /** Action when eventBtn pressed. */
    abstract void onEventBtn();

    /** Action when loadBtn pressed. */
    abstract void onLoadBtn();

    /** Action when resetBtn pressed. */
    abstract void onResetBtn();
    
    private JSeparator getJSeparator1() {
    	if(jSeparator1 == null) {
    		jSeparator1 = new JSeparator();
    	}
    	return jSeparator1;
    }
} // @jve:decl-index=0:visual-constraint="44,10"
